# -*- coding: iso-8859-1 -*-
"""
    GESTIKK
    Version 0.4

    Gestikk, a mouse gesture recognition application
    Copyright (C) 2007 Friedrich 'fred.reichbier' Weber (2007-10-08)

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""

VERSION = '0.4'

import sys
import os
import gettext
import locale
import math
import subprocess
import wx
import ConfigParser
import xml.etree.ElementTree as ETree

if not os.path.isdir(os.path.expanduser('~/.gestikk')):
    os.mkdir(os.path.expanduser('~/.gestikk'))
CONFIG_FILE = os.path.expanduser('~/.gestikk/gestikk.conf')
GESTURE_FILE = os.path.expanduser('~/.gestikk/gestures.xml')

def get_basedir():
    """ func to get the basedir, from exaile """
    basedir = os.path.dirname(os.path.realpath(__file__))
    if not os.path.exists(os.path.join(basedir, "gestikk.py")):
        if os.path.exists(os.path.join(os.getcwd(), "gestikk.py")):
            basedir = os.getcwd()
    return basedir

basedir = get_basedir()
sys.path.insert(0, basedir)
os.chdir(basedir)
BASEDIR = basedir
#print 'Basedir is', basedir

GPL = """		    GNU GENERAL PUBLIC LICENSE
		       Version 2, June 1991

 Copyright (C) 1989, 1991 Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 Everyone is permitted to copy and distribute verbatim copies
 of this license document, but changing it is not allowed.

			    Preamble

  The licenses for most software are designed to take away your
freedom to share and change it.  By contrast, the GNU General Public
License is intended to guarantee your freedom to share and change free
software--to make sure the software is free for all its users.  This
General Public License applies to most of the Free Software
Foundation's software and to any other program whose authors commit to
using it.  (Some other Free Software Foundation software is covered by
the GNU Lesser General Public License instead.)  You can apply it to
your programs, too.

  When we speak of free software, we are referring to freedom, not
price.  Our General Public Licenses are designed to make sure that you
have the freedom to distribute copies of free software (and charge for
this service if you wish), that you receive source code or can get it
if you want it, that you can change the software or use pieces of it
in new free programs; and that you know you can do these things.

  To protect your rights, we need to make restrictions that forbid
anyone to deny you these rights or to ask you to surrender the rights.
These restrictions translate to certain responsibilities for you if you
distribute copies of the software, or if you modify it.

  For example, if you distribute copies of such a program, whether
gratis or for a fee, you must give the recipients all the rights that
you have.  You must make sure that they, too, receive or can get the
source code.  And you must show them these terms so they know their
rights.

  We protect your rights with two steps: (1) copyright the software, and
(2) offer you this license which gives you legal permission to copy,
distribute and/or modify the software.

  Also, for each author's protection and ours, we want to make certain
that everyone understands that there is no warranty for this free
software.  If the software is modified by someone else and passed on, we
want its recipients to know that what they have is not the original, so
that any problems introduced by others will not reflect on the original
authors' reputations.

  Finally, any free program is threatened constantly by software
patents.  We wish to avoid the danger that redistributors of a free
program will individually obtain patent licenses, in effect making the
program proprietary.  To prevent this, we have made it clear that any
patent must be licensed for everyone's free use or not licensed at all.

  The precise terms and conditions for copying, distribution and
modification follow.

		    GNU GENERAL PUBLIC LICENSE
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. This License applies to any program or other work which contains
a notice placed by the copyright holder saying it may be distributed
under the terms of this General Public License.  The "Program", below,
refers to any such program or work, and a "work based on the Program"
means either the Program or any derivative work under copyright law:
that is to say, a work containing the Program or a portion of it,
either verbatim or with modifications and/or translated into another
language.  (Hereinafter, translation is included without limitation in
the term "modification".)  Each licensee is addressed as "you".

Activities other than copying, distribution and modification are not
covered by this License; they are outside its scope.  The act of
running the Program is not restricted, and the output from the Program
is covered only if its contents constitute a work based on the
Program (independent of having been made by running the Program).
Whether that is true depends on what the Program does.

  1. You may copy and distribute verbatim copies of the Program's
source code as you receive it, in any medium, provided that you
conspicuously and appropriately publish on each copy an appropriate
copyright notice and disclaimer of warranty; keep intact all the
notices that refer to this License and to the absence of any warranty;
and give any other recipients of the Program a copy of this License
along with the Program.

You may charge a fee for the physical act of transferring a copy, and
you may at your option offer warranty protection in exchange for a fee.

  2. You may modify your copy or copies of the Program or any portion
of it, thus forming a work based on the Program, and copy and
distribute such modifications or work under the terms of Section 1
above, provided that you also meet all of these conditions:

    a) You must cause the modified files to carry prominent notices
    stating that you changed the files and the date of any change.

    b) You must cause any work that you distribute or publish, that in
    whole or in part contains or is derived from the Program or any
    part thereof, to be licensed as a whole at no charge to all third
    parties under the terms of this License.

    c) If the modified program normally reads commands interactively
    when run, you must cause it, when started running for such
    interactive use in the most ordinary way, to print or display an
    announcement including an appropriate copyright notice and a
    notice that there is no warranty (or else, saying that you provide
    a warranty) and that users may redistribute the program under
    these conditions, and telling the user how to view a copy of this
    License.  (Exception: if the Program itself is interactive but
    does not normally print such an announcement, your work based on
    the Program is not required to print an announcement.)

These requirements apply to the modified work as a whole.  If
identifiable sections of that work are not derived from the Program,
and can be reasonably considered independent and separate works in
themselves, then this License, and its terms, do not apply to those
sections when you distribute them as separate works.  But when you
distribute the same sections as part of a whole which is a work based
on the Program, the distribution of the whole must be on the terms of
this License, whose permissions for other licensees extend to the
entire whole, and thus to each and every part regardless of who wrote it.

Thus, it is not the intent of this section to claim rights or contest
your rights to work written entirely by you; rather, the intent is to
exercise the right to control the distribution of derivative or
collective works based on the Program.

In addition, mere aggregation of another work not based on the Program
with the Program (or with a work based on the Program) on a volume of
a storage or distribution medium does not bring the other work under
the scope of this License.

  3. You may copy and distribute the Program (or a work based on it,
under Section 2) in object code or executable form under the terms of
Sections 1 and 2 above provided that you also do one of the following:

    a) Accompany it with the complete corresponding machine-readable
    source code, which must be distributed under the terms of Sections
    1 and 2 above on a medium customarily used for software interchange; or,

    b) Accompany it with a written offer, valid for at least three
    years, to give any third party, for a charge no more than your
    cost of physically performing source distribution, a complete
    machine-readable copy of the corresponding source code, to be
    distributed under the terms of Sections 1 and 2 above on a medium
    customarily used for software interchange; or,

    c) Accompany it with the information you received as to the offer
    to distribute corresponding source code.  (This alternative is
    allowed only for noncommercial distribution and only if you
    received the program in object code or executable form with such
    an offer, in accord with Subsection b above.)

The source code for a work means the preferred form of the work for
making modifications to it.  For an executable work, complete source
code means all the source code for all modules it contains, plus any
associated interface definition files, plus the scripts used to
control compilation and installation of the executable.  However, as a
special exception, the source code distributed need not include
anything that is normally distributed (in either source or binary
form) with the major components (compiler, kernel, and so on) of the
operating system on which the executable runs, unless that component
itself accompanies the executable.

If distribution of executable or object code is made by offering
access to copy from a designated place, then offering equivalent
access to copy the source code from the same place counts as
distribution of the source code, even though third parties are not
compelled to copy the source along with the object code.

  4. You may not copy, modify, sublicense, or distribute the Program
except as expressly provided under this License.  Any attempt
otherwise to copy, modify, sublicense or distribute the Program is
void, and will automatically terminate your rights under this License.
However, parties who have received copies, or rights, from you under
this License will not have their licenses terminated so long as such
parties remain in full compliance.

  5. You are not required to accept this License, since you have not
signed it.  However, nothing else grants you permission to modify or
distribute the Program or its derivative works.  These actions are
prohibited by law if you do not accept this License.  Therefore, by
modifying or distributing the Program (or any work based on the
Program), you indicate your acceptance of this License to do so, and
all its terms and conditions for copying, distributing or modifying
the Program or works based on it.

  6. Each time you redistribute the Program (or any work based on the
Program), the recipient automatically receives a license from the
original licensor to copy, distribute or modify the Program subject to
these terms and conditions.  You may not impose any further
restrictions on the recipients' exercise of the rights granted herein.
You are not responsible for enforcing compliance by third parties to
this License.

  7. If, as a consequence of a court judgment or allegation of patent
infringement or for any other reason (not limited to patent issues),
conditions are imposed on you (whether by court order, agreement or
otherwise) that contradict the conditions of this License, they do not
excuse you from the conditions of this License.  If you cannot
distribute so as to satisfy simultaneously your obligations under this
License and any other pertinent obligations, then as a consequence you
may not distribute the Program at all.  For example, if a patent
license would not permit royalty-free redistribution of the Program by
all those who receive copies directly or indirectly through you, then
the only way you could satisfy both it and this License would be to
refrain entirely from distribution of the Program.

If any portion of this section is held invalid or unenforceable under
any particular circumstance, the balance of the section is intended to
apply and the section as a whole is intended to apply in other
circumstances.

It is not the purpose of this section to induce you to infringe any
patents or other property right claims or to contest validity of any
such claims; this section has the sole purpose of protecting the
integrity of the free software distribution system, which is
implemented by public license practices.  Many people have made
generous contributions to the wide range of software distributed
through that system in reliance on consistent application of that
system; it is up to the author/donor to decide if he or she is willing
to distribute software through any other system and a licensee cannot
impose that choice.

This section is intended to make thoroughly clear what is believed to
be a consequence of the rest of this License.

  8. If the distribution and/or use of the Program is restricted in
certain countries either by patents or by copyrighted interfaces, the
original copyright holder who places the Program under this License
may add an explicit geographical distribution limitation excluding
those countries, so that distribution is permitted only in or among
countries not thus excluded.  In such case, this License incorporates
the limitation as if written in the body of this License.

  9. The Free Software Foundation may publish revised and/or new versions
of the General Public License from time to time.  Such new versions will
be similar in spirit to the present version, but may differ in detail to
address new problems or concerns.

Each version is given a distinguishing version number.  If the Program
specifies a version number of this License which applies to it and "any
later version", you have the option of following the terms and conditions
either of that version or of any later version published by the Free
Software Foundation.  If the Program does not specify a version number of
this License, you may choose any version ever published by the Free Software
Foundation.

  10. If you wish to incorporate parts of the Program into other free
programs whose distribution conditions are different, write to the author
to ask for permission.  For software which is copyrighted by the Free
Software Foundation, write to the Free Software Foundation; we sometimes
make exceptions for this.  Our decision will be guided by the two goals
of preserving the free status of all derivatives of our free software and
of promoting the sharing and reuse of software generally.

			    NO WARRANTY

  11. BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
REPAIR OR CORRECTION.

  12. IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES,
INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING
OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED
TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
POSSIBILITY OF SUCH DAMAGES.""" # this will be replaced with the `make_release.py` tool

def between(i, b1, b2):
    """ Is `i` between `b1` and `b2`? """
    return (i > b1 and i < b2)

def split_gesture(s):
    """ split gesture by ,; return list of string-tuples """
    l = []
    old = ''
    in_tuple = False
    for char in s:
        if char == '(':
            old += char
            in_tuple = True
        elif char == ')':
            old += char
            in_tuple = False
        elif char == ',' and not in_tuple:
            l.append(old)
            old = ''
        else:
            old += char
    if old:
        l.append(old)
    return l


def gesture2py(s):
    """ Convert a gesture to a python list with tuples """
    l = []
    for tup in split_gesture(s):
        l.append(str2tuple(tup))
    return l

def str2tuple(s):
    """Convert tuple-like strings to real tuples.
    e.g. '(1,2,3,4)' -> (1, 2, 3, 4)

    from http://www.dbforums.com/showpost.php?&p=5469452&postcount=15 - thanks :-)
    """
    if s[0] + s[-1] != "()":
        raise ValueError("Badly formatted string (missing brackets).")
    items = s[1:-1] # removes the leading and trailing brackets
    items = items.split(',')
    L = [int(x.strip()) for x in items] # clean up spaces, convert to ints
    return tuple(L)

def in_range(a, b, shw=60):
    """ Is `a` in range of `b` ?
        @param a: a point (tuple)
        @param b: a point (tuple)
        @param shw: the tolerance value
    """
    dx, dy = a[0] - b[0], a[1] - b[1]
    if abs(dx) > shw or abs(dy) > shw:
        return True
    else:
        return False

def calc_direction(a, b, shw=20):
    """ Calculate direction from pos1 (a) to pos2 (a), return tuple (x,y), e.g. (1,1)
        shw: as higher as more tolerant. recommended: ~ 20 """
    # NEW Method with angles (2007-12-28)
    # http://pics.ubuntu-projekte.de/bild.php/205,winkel2PL7C6.png
    # alpha = arctan(opposite/adjacent)
    # alpha = arctan(gegenkathete/ankathete)
    move_x, move_y = 0,0
    c = (b[0], a[1])
    ac = c[0] - a[0]
    bc = b[1] - c[1]
    alpha = math.degrees(math.atan2(bc, ac))
    #if abs(b[0]-a[0]) > shw and abs(b[1]-a[1]) > shw:
    if in_range(b, a, shw):
        if between(alpha, -15, 15):
            return (1, 0)
        elif between(alpha, 15, 75):
            return (1, 1)
        elif between(alpha, 75, 105):
            return (0, 1)
        elif between(alpha, 105, 165):
            return (-1, 1)
        elif alpha < -165 or alpha > 165:
            return (-1, 0)
        elif between(alpha, -165, -105):
            return (-1, -1)
        elif between(alpha, -105, -75):
            return (0, -1)
        elif between(alpha, -75, -15):
            return (1, -1)
        else:
            return (0,0)
    else:
        return (0,0)


def remove_duplicates(l):
    """ Remove upfollowing duplicates from the list """
    new= l[:]
    last_e = None
    for e in l:
        if e == last_e:
            new.remove(e)
        last_e = e
    return new

def dict_rev(d):
    """ return d 'reversed': keys are values, values are keys """
    r = {}
    for k,v in d.iteritems():
        r[v] = k
    return r

def format_action(action):
    """ Format an Action() human-readable """
    return _('Gesture %s: %s') % (action.gesture, action.get_formatted())

def get_language():
    """ get the preferred language, e.g. de """
    cp = ConfigParser.ConfigParser() # nicht gerade sauber, aber sonst fllt mir nichts ein
    cp.read(CONFIG_FILE)
    try:
        if cp.has_section('Preferences') \
            and cp.has_option('Preferences', 'language') \
            and cp.get('Preferences', 'language') != '':
            return cp.get('Preferences', 'language')
        else:
            return locale.getdefaultlocale()[0].split('_')[0]
    finally:
        del cp

def init_gettext():
    """ init gettext """
    gettext.install('gestikk')
    #lang = gettext.translation('gestikk', '/usr/share/locale', languges=[get_language()], fallback=True)
    # first in system's defaults
    try:
        lang = gettext.translation('gestikk', languages=[get_language()])
    # then in ./locale/
    except IOError:
        lang = gettext.translation('gestikk', os.path.join(BASEDIR, 'locale'), languages=[get_language()], fallback=True)
        # otherwise: fallback (english)
    return lang

def is_gestikk(pid):
    """ Test whether the process with the given pid is gestikk.
        It has to be started with python gestikk.py, gestikk or something similar.
        Works only with Unix/Linux """
    fcmdline = file('/proc/%d/cmdline' % pid, 'r')
    content = fcmdline.read()
    fcmdline.close()
    return 'gestikk' in content

def about_dialog_info():
    """ get wx about dialog information """
    info = wx.AboutDialogInfo()
    info.AddDeveloper("Friedrich 'fred.reichbier' Weber")
    info.AddTranslator("""CREDITS

 * English Translation by dejot
 * French Translation (WIP) by manatlan
""")
    info.SetCopyright('(C) 2007 Friedrich Weber')
    # get GPL
    info.SetLicense(GPL)
    info.SetName('Gestikk Mouse Gesture Recognition')
    info.SetIcon(wx.Icon(os.path.join(BASEDIR, 'media', 'gestikk-logo.png'), wx.BITMAP_TYPE_PNG))
    info.SetWebSite('http://projects.reichbier.de/gestikk/')
    return info

class ActionTemplate():
    """ Template class for writin actions into configuration file """
    def __init__(self, caction, help):
        """ Ident is the Action()
            help is a helptext"""
        self.caction = caction
        self.help = help

class Action():
    """ Action to execute; Prototyp-Class """
    def __init__(self, gesture, parameter):
        self.gesture = gesture

    def do(self):
        pass

    def get_formatted(self):
        """ Get description of the action as human-readable string """
        return ''

class Action_Command(Action):
    """ Aktion: Execute Command """
    def __init__(self, gesture, cmd):
        """ cmd: Command as a string """
        Action.__init__(self, gesture, cmd)
        self.action = 'cmd'
        self.parameter = cmd

    def do(self):
        """ tu es """
        action_split = self.parameter.split(' ')
        subprocess.Popen(action_split)

    def get_formatted(self):
        """ Get description of the action as human-readable string """
        return _('Start "%s"') % self.parameter


class Action_Key(Action):
    """ Aktion: Press key """
    def __init__(self, gesture, key):
        """ key: Key as X-Keysym, eg. 'alt+Left' """
        Action.__init__(self, gesture, key)
        self.action = 'key'
        self.parameter = key

    def do(self):
        """ tu es """
        subprocess.Popen(['xdotool', 'key', self.parameter], stdout=sys.stdout)

    def get_formatted(self):
        """ Get description of the action as human-readable string """
        return _('Key "%s"') % self.parameter

class GestureXMLReader:
    """ gestures.xml Parser Class """
    def __init__(self, xmlfile):
        self.xmlfilename = xmlfile
        self.tree = None
        self.gestures = {} # gesture expr:Action()

    def parse(self):
        """ Parse the XML-File
        """
        self.tree = ETree.parse(self.xmlfilename)
        self.root = self.tree.getroot()
        for node in self.root.getchildren():
            caction = ACTIONS[node.get('action')]
            gesture = node.get('gesture')
            self.gestures[gesture] = caction(gesture, node.get('parameter'))

        return self.gestures

class GestureXMLWriter:
    """ gestures.xml Writer class """
    def __init__(self, gestures, dest):
        # gesture expr:Action()
        self.dest = dest
        self.gestures = gestures

    def write(self):
        """ write it """
        root = ETree.Element('gestikk')

        for gesture in self.gestures.values():
            node = ETree.SubElement(root, 'gesture')
            node.set('gesture', gesture.gesture)
            node.set('action', gesture.action)
            node.set('parameter', gesture.parameter)

        tree = ETree.ElementTree(root)
        tree.write(self.dest)

class Logic():
    """ class for saving the logic (of gestures and preferences) """
    def __init__(self):
        # Config Parser
        self.config = ConfigParser.ConfigParser()
        self.config_file = CONFIG_FILE # todo
        self.config.read(self.config_file)
        # Gesture Parser
        if os.path.isfile(GESTURE_FILE):
            self.gestures = GestureXMLReader(GESTURE_FILE).parse() # gesture expr: Action()
        else:
            self.gestures = {}

        self.action_list = {_('Command'):
            ActionTemplate(Action_Command, _('Execute a command (eg. gksu scite)')),
            _('Keypress'):
            ActionTemplate(Action_Key, _('Press a key combination (eg. alt+Left)'))}

        # switcher
        self.gesture_switchers_txt = {_('Left Mouse Button'): 0,
                                _('Middle Mouse Button'): 1,
                                _('Right Mouse Button'): 2,
                                _('Control Button Down'): 3,
                                _('Alt Button Down'): 4,
                                _('Shift Button Down'): 5,
                                _('Cmd Button Down'): 6,
                                _('Meta Button Down'): 7}
        self.gesture_switchers = ['leftdown', 'middledown', 'rightdown', 'ctrldown', 'altdown', 'shiftdown', 'cmddown', 'metadown']

        self.current_switcher = self.config_get_option('Preferences', 'switcher', 'rightdown')
        self.current_switcher_txt = dict_rev(self.gesture_switchers_txt)[self.gesture_switchers.index(self.current_switcher)]


    def add_gesture(self, action):
        """ add the gesture """
        self.gestures[action.gesture] = action

    def config_get_option(self, section, option, default=''):
        """ Fetch option """
        if self.config.has_section(section) and self.config.has_option(section, option):
            return self.config.get(section, option)
        else:
            return default

    def config_set_option(self, section, option, value):
        """ Setze Option, lege ggf. Sektion neu an """
        if not self.config.has_section(section):
            self.config.add_section(section)
        self.config.set(section, option, value)

    def config_write(self):
        """ Write configuration file """
        fp = file(self.config_file, 'w')
        self.config.write(fp)
        fp.close()

    def save_gestures(self):
        """ Write gesture file """
        GestureXMLWriter(self.gestures, GESTURE_FILE).write()

    def reload_gestures(self):
        """ Reload the gesture file """
        if os.path.isfile(GESTURE_FILE):
            self.gestures = GestureXMLReader(GESTURE_FILE).parse() # gesture expr: Action()
        else:
            self.gestures = {}

    def reload_config(self):
        """ Reload configuration file """
        del self.config
        self.config = ConfigParser.ConfigParser()
        self.config_file = CONFIG_FILE # todo
        self.config.read(self.config_file)
        # switcher
        self.current_switcher = self.config_get_option('Preferences', 'switcher', 'rightdown')
        self.current_switcher_txt = dict_rev(self.gesture_switchers_txt)[self.gesture_switchers.index(self.current_switcher)]


class ViewGestureDialog(wx.Dialog):
    """ a dialog for viewing a gesture """
    def __init__(self, gesture, parent = None, id = -1, title='', size=wx.Size(640,480)):
        """ `gesture` is a gestikk gesture tuple list """
        wx.Dialog.__init__(self, parent, id, title, size=size)
        self.SetBackgroundColour("white")
        panel = wx.Panel(self)
        panel.Bind(wx.EVT_PAINT, self.on_panel_paint)
        #self.gesture = [(t[0]*100+200, t[1]*100+200) for t in gesture]
        self.gesture = gesture
        self.panel = panel
        self.set_size = True

    def get_09(self):
        """ calculate the 0,0 point """
        point0 = [0, 0] # the zero point
        point9 = [0, 0] # the last point
        # imagine a rectangle:
        # P0 ------------------------
        # |                          |
        # |                          |
        # --------------------------P9
        # P0: point0
        # P9: point9
        # ... easy, isn't it? :-P
        point = point0[:]
        first = True
        for gesture in self.gesture:
            if first:
                new_point = (point[0]+gesture[0]*150-10, point[1]+gesture[1]*150)
            else:
                new_point = (point[0]+gesture[0]*150, point[1]+gesture[1]*150)
            # Set Point0
            if new_point[0] < point0[0]:
                point0[0] = new_point[0]
            if new_point[1] < point0[1]:
                point0[1] = new_point[1]
            # Set Point9
            if abs(new_point[0]) > point9[0]:
                point9[0] = abs(new_point[0])
            if abs(new_point[1]) > point9[1]:
                point9[1] = abs(new_point[1])
            point = new_point
            first = False
        return point0, point9

    def on_panel_paint(self, event = None):
        dc = wx.PaintDC(self.panel)

        dc.SetBackground(wx.Brush("white", wx.SOLID))
        dc.Clear()

        # GraphicsContext erstellen (damit sehen die Bilder besser aus)
        gc = wx.GraphicsContext_Create(dc)

        # Blauen Stift zuweisen
        color = wx.Colour(0, 0, 255)
        pen = wx.Pen(color, 2, wx.SOLID)
        gc.SetPen(pen)

        point0, point9 = self.get_09()
        point = (-point0[0]+50, -point0[1]+50)
        # draw the start point
        gc.DrawEllipse(point[0]-2, point[1]-2, 4, 4)

        first = True
        for gesture in self.gesture:
            if first:
                new_point = (point[0]+gesture[0]*150-10, point[1]+gesture[1]*150)
            else:
                new_point = (point[0]+gesture[0]*150, point[1]+gesture[1]*150)
            gc.DrawLines([point, new_point])
            point = new_point
            first = False
            # set the color
            if color.Blue() >= 70:
                color.Set(0, 0, color.Blue()-70)
                pen.SetColour(color)
                gc.SetPen(pen)
        # resize the window
        if self.set_size:
            wx.CallAfter(self.SetClientSizeWH, point9[0]+100, point9[1]+100)
            self.set_size = False


ACTIONS = {'cmd':Action_Command, 'key':Action_Key}
_ = init_gettext().gettext
