#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-
"""
    GESTIKK
    Version 0.4

    Gestikk, a mouse gesture recognition application
    Copyright (C) 2007 Friedrich 'fred.reichbier' Weber (2007-10-08)

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""

from __future__ import division
import os
import os.path
import signal
import wx
import time
from thread import start_new_thread
from gestikk_tools import *

_ = init_gettext().gettext

if not os.path.isdir(os.path.expanduser('~/.gestikk')): # Create directory is it doesnt exist
    os.mkdir(os.path.expanduser('~/.gestikk'))
CONFIG_FILE = os.path.expanduser('~/.gestikk/gestures.conf') # TODO

wx.SetDefaultPyEncoding("iso-8859-1")

class ActionDialog(wx.Dialog):
    """ Choose Action - Dialogue """
    def __init__(self, logic, parent=None, id=-1, title=_('Action'), size=wx.Size(550, 430)):
        wx.Dialog.__init__(self, parent, id, title, size=size)
        self.logic = logic

        sizer = wx.BoxSizer(wx.VERTICAL)
        lbl_help = wx.StaticText(self, label=_('Choose now which action should be executed.'))
        sizer.Add(lbl_help)
        panel = wx.Panel(self)
        # now the interestin part #
        grid_sizer = wx.GridSizer(rows=2, cols=2)
        # list
        self.list_keys = self.logic.action_list.keys()
        lst_actions = wx.ListBox(panel, choices=self.list_keys, size=wx.Size(150,200))
        grid_sizer.Add(lst_actions)
        self.lst_actions = lst_actions
        lst_actions.Bind(wx.EVT_LISTBOX, self.evt_list)
        # # # # # # #
        sizer2 = wx.BoxSizer(wx.VERTICAL)
        panel2 = wx.Panel(panel)
        # input box
        input = wx.TextCtrl(panel2, size=wx.Size(200, 25))
        self.input = input
        sizer2.Add(input, 0, wx.ALIGN_RIGHT | wx.ALL)
        # helptext
        lbl_help = wx.StaticText(panel2, size=wx.Size(200,300))
        self.lbl_help = lbl_help
        sizer2.Add(lbl_help, 0, wx.ALL, 5)
        # # # # # # #
        panel2.SetSizer(sizer2)
        # # # # # # #
        grid_sizer.Add(panel2)
        # # # # # # # # # # # # # #
        panel.SetSizer(grid_sizer)
        # # # #
        button = wx.Button(self, label=_('Create'))
        button.Bind(wx.EVT_BUTTON, self.evt_save)
        sizer.Add(panel, 0, wx.ALL, 5)
        sizer.Add(button)
        # #
        self.SetSizer(sizer)

        self.caction = None

    def evt_save(self, evt):
        """ save """
        if self.caction:
            self.value = self.caction('', self.input.GetValue()) # Action(), blank gesture placeholder
        self.EndModal(wx.ID_OK)

    def evt_list(self, evt):
        """ something new chosen, show helptext """
        try:
            template = self.logic.action_list[self.list_keys[self.lst_actions.GetSelections()[0]]]
            self.caction = template.caction
        except IndexError:
            print _('Small error, that hasn\'t much to say (1)')
            return
        # show whelptext
        self.lbl_help.SetLabel(template.help)
        self.lbl_help.Wrap(200)
        evt.Skip()


class NewGestureDialog(wx.Dialog):
    """ Neue Mausgeste - Dialog """
    def __init__(self, logic, parent=None, id=-1, title=_('New Gesture'), size=wx.Size(550, 430)):
        wx.Dialog.__init__(self, parent, id, title, size=size)
        self.logic = logic

        sizer = wx.BoxSizer(wx.VERTICAL)
        lbl_do = wx.StaticText(self, label=_('Draw the gesture (big one!) in the window while pressing the right mouse button.\nAttention! An already drawed action will be overwritten!'))
        sizer.Add(lbl_do)
        self.lbl_do = lbl_do

        self.gesture_positions = []

        self.Bind(wx.EVT_CLOSE, self.evt_close)
        self.is_looping = 1 # is set to 0 if thread should be closed
        self.gestures = []
        start_new_thread(self.mouse_gesture_thread, ())

    def evt_close(self, evt):
        """ Closing: Clsoe thread if not already closed (like an emergency brake :-)) """
        self.is_looping = 0
        time.sleep(0.26)
        evt.Skip()

    def mouse_gesture_thread(self):
        """ Thread for mouse gesture recognition """
        while self.is_looping:
            wx.CallAfter(wx.YieldIfNeeded)
            wx.CallAfter(self.check_mouse_position)
            time.sleep(0.25)

    def check_gestures(self):
        """ Check both gestures in self.gestures """
        if self.gestures[0] != self.gestures[1]: # Bot the same
            self.lbl_do.SetLabel(_('Unfortunately the gestures don\'t match.\nTry again please!'))
            self.gestures = []
        else:
            #dlg = wx.TextEntryDialog(
            #        self, _('Now enter the command that will be executed by this gesture (eg. gksu gimp).'),
            #        _('Aktion'), '')
            dlg = ActionDialog(self.logic, self)
            if dlg.ShowModal() == wx.ID_OK:
                action = dlg.value
                action.gesture = self.gestures[0][1:-1]
                self.logic.add_gesture(action)
                wx.CallAfter(self.Close)
            else:
                self.lbl_do.SetLabel(_('If you want to, try again.\nElse just close this dialogue.'))
                self.gestures = []
            dlg.Destroy()


    def check_mouse_position(self):
        """ Check position of mouse pointer """
        pos = wx.GetMousePosition()
        mstate = wx.GetMouseState().RightDown()
        if mstate: # gesture because right button is pressed
            self.gesture_positions.append(pos)
        else: # Not longer pressed
            if self.gesture_positions: # gesture finished
                # Pfad errechnen
                gesture = []
                tolerance = int(self.logic.config_get_option('Preferences', 'tolerance', '25'))
                for gid in range(1, len(self.gesture_positions)): # let first out
                    gesture.append(calc_direction(self.gesture_positions[gid-1], \
                                                self.gesture_positions[gid], tolerance))
                # Remove zeros
                while (0,0) in gesture:
                    gesture.remove((0,0))
                # Delete duplicates
                gesture = remove_duplicates(gesture)
                self.gesture_positions = []
                # Is a command assigned to the gesture?
                gesture_repr = repr(gesture).replace(' ', '') # w/o spaces
                self.gestures.append(gesture_repr)
                if gesture_repr == '[]': # leer?
                    self.lbl_do.SetLabel(_('That was too fast.\nPlease do it again.'))
                    self.gestures = []
                    self.gesture_positions = []
                    return
                if len(self.gestures) == 1:
                    # first gesture: encourage the user
                    self.lbl_do.SetLabel(_('Now again please so that I can see,\nif you misdrawed it :-)'))
                else:
                    # Second gesture finished, check it
                    self.check_gestures()
                if DEBUG:
                    print 'Gesture: %s' % gesture_repr

class MGConfigFrame(wx.Frame):
    def __init__(self, logic, parent = None, id = -1, title = _("Gestikk - Configuration"), size=wx.Size(640,480)):
        wx.Frame.__init__(self, parent, id, title, size=size)
        self.logic = logic
        #

        sizer = wx.BoxSizer(wx.VERTICAL)
        #sizer = wx.BoxSizer(wx.VERTICAL)
        self.SetSizer(sizer)
        self.notebook = wx.Notebook(self)
        sizer.Add(self.notebook, proportion=1, flag=wx.ALL|wx.EXPAND) # dont know if this is correct but it works
        # --- Panel for buttons
        pbuttons = wx.Panel(self)
        sbuttons = wx.BoxSizer(wx.HORIZONTAL)
        #sbuttons = wx.CreateButtonSizer(wx.OK | wx.CANCEL)
        # 'Save' - Button
        btnsave = wx.Button(pbuttons, id=wx.ID_APPLY)
        btnsave.Bind(wx.EVT_BUTTON, self.save_all)
        sbuttons.Add(btnsave, 0, wx.ALL)
        # spacer
        sbuttons.AddStretchSpacer()

        # 'About' - Button
        btnabout = wx.Button(pbuttons, id=wx.ID_ABOUT)
        btnabout.Bind(wx.EVT_BUTTON, self.evt_about)
        sbuttons.Add(btnabout, 0, wx.ALL)
        # 'Close' - Button
        btnclose = wx.Button(pbuttons, id=wx.ID_CANCEL)
        btnclose.Bind(wx.EVT_BUTTON, self.evt_btn_close)
        sbuttons.Add(btnclose, 0, wx.ALL)
        # ---
        pbuttons.SetSizer(sbuttons)
        sizer.Add(pbuttons,0, wx.EXPAND | wx.ALL, 5)
        # ----- Seite 1:
        self.notebook.AddPage(self.get_page_preferences(), _('Preferences'))
        # ----- Seite 2:
        self.notebook.AddPage(self.get_page_gestures(), _('Gestures'))


    def get_option(self, section, option, default=''):
        """ Wrapper for self.logic.config_get_option """
        return self.logic.config_get_option(section, option, default)

    def save_preferences(self):
        """ Save the preferences """
        self.logic.config_set_option('Preferences', 'tolerance', self.page_preferences.v_tolerance.GetValue())
        self.logic.config_set_option('Preferences', 'language', self.page_preferences.v_lang.GetValue())
        self.logic.config_set_option('Preferences', 'switcher', self.logic.gesture_switchers[self.logic.gesture_switchers_txt[self.page_preferences.v_gswitch.GetValue()]])

    def save_gestures(self):
        """ Save the gestures """
        self.logic.save_gestures()

    def evt_btn_close(self, evt):
        self.Close()

    def save_all(self, evt):
        """ Save everything """
        self.save_preferences()
        self.save_gestures()
        self.logic.config_write()
        # send signal to the gestikk main process (the parent process)
        ppid = os.getppid()
        if is_gestikk(ppid):
            os.kill(ppid, signal.SIGUSR1)

    def get_page_preferences(self):
        """ Put out panel 'Preferences' """
        #--- Sizer u.a.
        page = wx.Panel(self.notebook)
        sizer = wx.GridSizer(rows=1, cols=2)
        page.SetSizer(sizer)
        #--- Inhalt
        # - Toleranz
        lbl_tolerance = wx.StaticText(page, label=_('Tolerance (as higher as more tolerant)\nRecommended: ~ 20'))
        sizer.Add(lbl_tolerance)
        tolerance = wx.SpinCtrl(page, max=100, initial=int(self.get_option('Preferences', 'tolerance', '20')))
        sizer.Add(tolerance)
        page.v_tolerance = tolerance
        # - Sprache
        lbl_lang = wx.StaticText(page, label=_('Default Language (e.g. de)\nIf empty, read it from system\'s default'))
        sizer.Add(lbl_lang)
        lang = wx.TextCtrl(page)
        lang.SetValue(self.get_option('Preferences', 'language', ''))
        sizer.Add(lang)
        page.v_lang = lang
        # - Aktion
        # - Sprache
        lbl_gswitch = wx.StaticText(page, label=_('Gesture switch action'))
        sizer.Add(lbl_gswitch)
        gswitch = wx.ComboBox(page, value=self.logic.current_switcher_txt, choices=self.logic.gesture_switchers_txt.keys())
        sizer.Add(gswitch)
        page.v_gswitch = gswitch
        #--- Fertig
        self.page_preferences = page
        return page

    def fill_gesture_list(self):
        """ Fill list of gestures """
        self.page_gestures.v_glist.Clear()
        i = 0
        for (gesture, action) in self.logic.gestures.iteritems():
            self.page_gestures.v_glist.Append(format_action(action))
            self.page_gestures.v_glist.SetClientData(i, action)
            i += 1

    def get_page_gestures(self):
        """ Put out panel 'Gestures' """
         #--- Sizer and others
        page = wx.Panel(self.notebook)
        sizer = wx.BoxSizer(wx.VERTICAL)
        page.SetSizer(sizer)
        #--- Content
        # - list
        lbl_glist = wx.StaticText(page, label=_('List of all gestures'))
        sizer.Add(lbl_glist)
        glist = wx.ListBox(page)
        # fill list
        sizer.Add(glist, proportion=1, flag=wx.EXPAND | wx.ALL)
        page.v_glist = glist
        self.page_gestures = page
        self.fill_gesture_list()
        # new sizer
        panel2 = wx.Panel(page)
        sizer2 = wx.BoxSizer(wx.HORIZONTAL)
        # - new gesture
        new_gesture = wx.Button(panel2, id=wx.ID_ADD)
        sizer2.Add(new_gesture)
        new_gesture.Bind(wx.EVT_BUTTON, self.evt_new_gesture)
        # - view gesture
        view_gesture = wx.Button(panel2, id=wx.ID_ANY, label=_('View gesture'))
        sizer2.Add(view_gesture)
        view_gesture.Bind(wx.EVT_BUTTON, self.evt_view_gesture)
        # - Delete gesture
        del_gesture = wx.Button(panel2, id=wx.ID_DELETE)
        sizer2.Add(del_gesture)
        del_gesture.Bind(wx.EVT_BUTTON, self.evt_del_gesture)
        #--- Ready
        # new sizer
        panel2.SetSizer(sizer2)
        sizer.Add(panel2)
        return page

    def evt_new_gesture(self, evt):
        """ New gesture """
        diag = NewGestureDialog(self.logic)
        diag.ShowModal()
        # Update the list
        self.fill_gesture_list()
        diag.Destroy()

    def evt_view_gesture(self, evt):
        """ View gesture """
        gesture = self.page_gestures.v_glist.GetClientData(self.page_gestures.v_glist.GetSelection())
        if gesture:
            diag = ViewGestureDialog(gesture2py(gesture.gesture))
            diag.SetTitle(_('View gesture'))
            diag.ShowModal()
            diag.Destroy()

    def evt_del_gesture(self, evt):
        """ delete gesture """
        gesture = self.page_gestures.v_glist.GetClientData(self.page_gestures.v_glist.GetSelection())
        if gesture:
            # delete
            del self.logic.gestures[gesture.gesture]
            self.fill_gesture_list()

    def evt_about(self, evt):
        """ Show about dialog """
        wx.AboutBox(about_dialog_info())

def main(debug):
    """
        :param debug: Debug mode on?
    """
    global DEBUG
    DEBUG = debug
    app = wx.PySimpleApp()
    logic = Logic()
    diag = MGConfigFrame(logic)
    diag.Show(True)
    app.SetTopWindow(diag)
    app.MainLoop()

if __name__ == '__main__':
    main(False)